/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.stdinc;

import core.stdc.stdlib;
import core.stdc.ctype;
import core.stdc.string;
import core.stdc.stdio;
import org.libsdl.config;

uint SDL_arraysize(T)(T[] array)
{
	return array.length;
}

alias SDL_arraysize SDL_TABLESIZE;

enum SDL_bool
{
	SDL_FALSE = 0,
	SDL_TRUE  = 1,
}

alias SDL_bool.SDL_FALSE SDL_FALSE;
alias SDL_bool.SDL_TRUE SDL_TRUE;

alias int8_t   Sint8;
alias uint8_t  Uint8;
alias int16_t  Sint16;
alias uint16_t Uint16;
alias int32_t  Sint32;
alias uint32_t Uint32;
alias int64_t  Sint64;
alias uint64_t Uint64;



alias malloc SDL_malloc;
alias calloc SDL_calloc;
alias realloc SDL_realloc;
alias free SDL_free;
alias alloca SDL_stack_alloc;
void SDL_stack_free(void* pt) {}
alias getenv SDL_getenv;

extern (C)
{
	int SDL_putenv(const(char)* variable);
}

alias qsort SDL_qsort;
alias abs SDL_abs;

T SDL_min(T)(T x, T y)
{
    return x < y ? x : y;
}

T SDL_max(T)(T x, T y)
{
    return x > y ? x : y;
}

alias isdigit SDL_isdigit;
alias isspace SDL_isspace;
alias toupper SDL_toupper;
alias tolower SDL_tolower;

alias memset SDL_memset;

void SDL_memset4(void* dst, uint val, uint len)
{
	if (len == 0)
		return;

	uint count = (len + 3) / 4;
	uint* p = cast(uint*)dst;

	switch (len % 4)
	{
	case 3:
		*p++ = val;
	case 2:
		*p++ = val;
	case 1:
		*p++ = val;
		--count;
	default:
	}

	for (uint i; i < count; ++i)
	{
		*p++ = val;
		*p++ = val;
		*p++ = val;
		*p++ = val;
	}
}

alias memcpy SDL_memcpy;

void SDL_memcpy4(void* dst, void* src, uint len)
{
	SDL_memcpy(dst, src, len << 2);
}

extern (C)
{
	void* SDL_revcpy(void *dst, const(void)* src, size_t len);
}

alias memmove SDL_memmove;
alias memcmp SDL_memcmp;
alias strlen SDL_strlen;

extern (C)
{
	size_t SDL_strlcpy(char* dst, const(char)* src, size_t maxlen);
	size_t SDL_strlcat(char* dst, const(char)* src, size_t maxlen);
	char* SDL_strdup(const(char)* str);
	char* SDL_strrev(char* str);
	char* SDL_strupr(char* str);
	char* SDL_strlwr(char* str);
}

alias strchr SDL_strchr;
alias strrchr SDL_strrchr;
alias strstr SDL_strstr;

extern (C)
{
	char* SDL_ltoa(int value, char* str, int radix);
}

alias SDL_ltoa SDL_itoa;

extern (C)
{
	char* SDL_ultoa(uint value, char* str, int radix);
}

alias SDL_ultoa SDL_uitoa;

extern (C)
{
	int SDL_strtol(const(char)* str, char** endp, int base);
	uint SDL_strtoul(const(char)* str, char** endp, int base);
}

extern (C)
{
	char* SDL_lltoa(Sint64 value, char* str, int radix);
	char* SDL_ulltoa(Uint64 value, char* str, int radix);
	Sint64 SDL_strtoll(const(char)* str, char** endp, int base);
	Uint64 SDL_strtoull(const(char)* str, char** endp, int base);
	double SDL_strtod(const(char)* str, char** endp);
}

int SDL_atoi(const(char)* str)
{
	return SDL_strtol(str, null, 0);
}

float SDL_atof(const(char)* str)
{
	return SDL_strtod(str, null);
}

alias strcmp SDL_strcmp;
alias strncmp SDL_strncmp;

extern (C)
{
	int SDL_strcasecmp(const(char)* str1, const(char)* str2);
	int SDL_strncasecmp(const(char)* str1, const(char)* str2, size_t maxlen);
}

alias sscanf SDL_sscanf;
alias snprintf SDL_snprintf;
alias vsnprintf SDL_vsnprintf;

immutable size_t SDL_ICONV_ERROR  = -1;
immutable size_t SDL_ICONV_E2BIG  = -2;
immutable size_t SDL_ICONV_EILSEQ = -3;
immutable size_t SDL_ICONV_EINVAL = -4;

struct _SDL_iconv_t;
alias _SDL_iconv_t* SDL_iconv_t;

extern (C)
{
	SDL_iconv_t SDL_iconv_open(const(char)* tocode, const(char)* fromcode);
	int SDL_iconv_close(SDL_iconv_t cd);
	size_t SDL_iconv(SDL_iconv_t cd, const(char)** inbuf, size_t* inbytesleft, char** outbuf, size_t* outbytesleft);
	char* SDL_iconv_string(const(char)* tocode, const(char)* fromcode, const(char)* inbuf, size_t inbytesleft);
}

char* SDL_iconv_utf8_locale(const(char)* inbuf)
{
	return SDL_iconv_string("", "UTF-8", inbuf, SDL_strlen(inbuf) + 1);
}

wchar* SDL_iconv_utf8_ucs2(const(char)* inbuf)
{
	return cast(wchar*)SDL_iconv_string("UCS-2", "UTF-8", inbuf, SDL_strlen(inbuf) + 1);
}

dchar* SDL_iconv_utf8_ucs4(const(char)* inbuf)
{
	return cast(dchar*)SDL_iconv_string("UCS-4", "UTF-8", inbuf, SDL_strlen(inbuf) + 1);
}

